@@ -1,5 +1,6 @@
 """Base class for all bounds."""
 from abc import abstractmethod
+from typing import Union
 
 from gemd.entity.dict_serializable import DictSerializable
 
@@ -8,14 +9,15 @@ class BaseBounds(DictSerializable):
     """Base class for bounds, including RealBounds and CategoricalBounds."""
 
     @abstractmethod
-    def contains(self, bounds):
+    def contains(self, bounds: Union["BaseBounds", "BaseValue"]):
         """
         Check if another bounds is contained within this bounds.
 
         Parameters
         ----------
-        bounds: BaseBounds
-            Other bounds object to check.
+        bounds: Union[BaseBounds, BaseValue]
+            Other bounds object to check.  If it's a Value object, check against
+            the smallest compatible bounds, as returned by the
 
         Returns
         -------
@@ -23,8 +25,12 @@ class BaseBounds(DictSerializable):
             True if any value that validates true for bounds also validates true for this
 
         """
+        from gemd.entity.value.base_value import BaseValue
+
         if bounds is None:
             return False
+        if isinstance(bounds, BaseValue):
+            bounds = bounds._to_bounds()
         if isinstance(bounds, BaseBounds):
             return True
         raise TypeError('{} is not a Bounds object'.format(bounds))
