@@ -24,14 +24,16 @@ class MaterialSpec(BaseObject, HasTemplate):
         for filtering and discoverability.
     notes: str, optional
         Long-form notes about the material spec.
-    process: ProcessSpec
+    process: :class:`ProcessSpec <gemd.entity.object.process_spec.ProcessSpec>`
         Process that produces this material.
-    properties: List[PropertyAndConditions], optional
+    properties: List[:class:`PropertyAndConditions\
+    <gemd.entity.attribute.property_and_conditions.PropertyAndConditions>`], optional
         Properties of the material, along with an optional list of conditions under which
         those properties are measured.
-    template: MaterialTemplate, optional
+    template: :class:`MaterialTemplate\
+    <gemd.entity.template.material_template.MaterialTemplate>`, optional
         A template bounding the valid values for this material's properties.
-    file_links: List[FileLink], optional
+    file_links: List[:class:`FileLink <gemd.entity.file_link.FileLink>`], optional
         Links to associated files, with resource paths into the files API.
 
     """
